import os
import re
import codecs
import glob
from bs4 import BeautifulSoup
from argparse import ArgumentParser

try:
    from tqdm import trange
except:
    print("tqdm not found, progress won't be shown")
    trange = None

C_RED = "\033[91m"
C_YELLOW = "\033[93m"
C_GREEN = "\033[92m"
C_BLUE = "\033[94m"
C_BOLD = "\033[1m"
C_ENDC = "\033[0m"

p = ArgumentParser()
p.add_argument("--verbose", "-v", action="store_true")
args = p.parse_args()

files = glob.glob(os.path.join("try26", "animetranscripts.wikispaces.com", "*.html"))
files.sort()
good_data = 0
questionable_data = 0

line_pattern = re.compile("[A-Za-z][A-Za-z0-9\-\_\. ]*: ")
colon_pattern = re.compile("(?<=[A-Za-z\.]) +(?=:)")
colon2_pattern = re.compile("(?<=:) *\n")
text_pattern = re.compile("(?<=[\?\.!])\n(?=[^:]+ )")
text2_pattern = re.compile("(?<=[\?\.!] )\n(?=[^:]+ )")
text3_pattern = re.compile("(?<=[\?\.!]  )\n(?=[^:]+ )")
newline_pattern = re.compile("(?<=[\?\.,!a-z]) *\n(?=[^:]+( |\n))")
newline2_pattern = re.compile("\n *")
whitespace_pattern = re.compile("(?<=:) *\n* *\n* *")
whitespace2_pattern = re.compile(" {2,}")
japanese_pattern = re.compile("[゠-ヿぁ-ゟ]")
link_pattern = re.compile("https?:")
title_pattern = re.compile("(?<=[a-z!.\?])(?=[A-Z0-9])")
title_whitespace_pattern = re.compile(" {2,}")
punctuation_pattern = re.compile("(?<=[\?\.!])(?=[A-Z])")
raw_sub_pattern = re.compile("Dialogue:|Format:")
cleanup_pattern = re.compile("(\[|<|--) *(OP|ED|Eye ?catch|Opening Song|Preview) *(\]|>|--)", re.I)
cleanup2_pattern = re.compile("-{2,} *-{2,} *(?=\n)")
cleanup3_pattern = re.compile("-{5,}[^-]-{5,}")
cleanup4_pattern = re.compile("-{3,} *(?=\n)")
cleanup5_pattern = re.compile("-{4,}[^\n]+(?=\n)")
cleanup6_pattern = re.compile("-{3,}( |\[)[^\n]+(?=\n)")
cleanup7_pattern = re.compile("- - -[^\n]+(?=\n)")
numbers_pattern = re.compile("[0-9]{1,2}-[0-9]{1,2}\.[^\n]*")
double_line_pattern = re.compile(r"(?<=[\.\,\?\!])\x20(?=[\(\)A-Za-z\' \&\-]+:)")
l_pattern = re.compile(r"(?<=[ \"])l(?=[ ])")
l_pattern2 = re.compile(r"(?<=[ \"])l'm")
l_pattern3 = re.compile(r"(?<=[ \"])l'd")
l_pattern4 = re.compile(r"(?<=[ \"])lt")
l_pattern5 = re.compile(r"(?<=[ \"])lf")
l_pattern6 = re.compile(r"(?<=[ \"])ln")
l_pattern7 = re.compile(r"(?<=[ \"])ls")
bad_filename_characters = re.compile("[?<>\\:*|\"]")

# GOOD
good_files = open("good_files.txt").read().strip().split("\n")
questionable_files = open("incomplete_files.txt").read().strip().split("\n")

# BAD
bad_files = [
    "Mobile Suit Gundam Unicorn Day of the Unicorn",
    u"When They Cry - ひぐらしのなく頃に",
    "Transcripts in English",
    
]
i = 0
#for filename in files:
if trange is None:
    range_iter = range(len(files))
else:
    range_iter = trange(len(files)-1)

for i in range_iter:
    i += 1
    filename = files[i]
    doc = BeautifulSoup(codecs.decode(open(filename, "rb").read(), "utf-8", "ignore"), "lxml")
    questionable = False
    try:
        title = doc.find("h1", attrs={"class":"pageTitle"}).text.replace(">", "")
        title = title_pattern.sub(" ", title)
        title = title_whitespace_pattern.sub(" ", title).strip()
        text = doc.find("div", attrs={"class":"wikiPage"}).text
        
        text = colon_pattern.sub("", text) # fix spaces between names and colon
        text = colon2_pattern.sub(" ", text) # fix newlines after name
        
        text = text_pattern.sub(" ", text) # combine multiple lines
        text = text2_pattern.sub(" ", text)
        text = text3_pattern.sub(" ", text)
        
        text = newline_pattern.sub(" ", text) # join lines
        text = newline2_pattern.sub("\n", text) # fix screwy newlines
        #text = whitespace_pattern.sub(" ", text) # fix screwy whitespace
        text = whitespace2_pattern.sub(" ", text)
        #text = punctuation_pattern.sub(" ", text) # fix missing whitespace
        
        
        #text = text.replace("- -", "--")
        text = cleanup_pattern.sub("", text)
        text = cleanup2_pattern.sub("", text)
        text = cleanup3_pattern.sub("", text)
        text = cleanup4_pattern.sub("", text)
        text = cleanup5_pattern.sub("", text)
        text = cleanup6_pattern.sub("", text)
        text = cleanup7_pattern.sub("", text)
        text = numbers_pattern.sub("", text)
        
        text = double_line_pattern.sub("\n", text)

        text = l_pattern.sub("I ", text)
        text = l_pattern2.sub("I'm", text)
        text = l_pattern3.sub("I'd", text)
        text = l_pattern4.sub("It", text)
        text = l_pattern5.sub("If", text)
        text = l_pattern6.sub("In", text)
        text = l_pattern7.sub("Is", text)
        #print(text)
        
        output = ""
        for line in text.split("\n"):
            if len(line) < 5: continue
            if line_pattern.match(line) is None or japanese_pattern.search(line) or link_pattern.search(line) or raw_sub_pattern.match(line):
                continue
            if len(line) > 300*6:
                #if args.verbose: print(line)
                #if args.verbose: print(len(line), "> 300*6", filename)
                questionable = True
            if line.find("\n") != -1:
                #if args.verbose: print(line.replace("\n", "XXX"))
                raise SystemExit
            output += line.strip() + "\n"
        #if args.verbose: print("TITLE:", title)
        #if args.verbose: print(filename)
        if questionable:
            questionable_data += len(output)
        else:
            good_data += len(output)
        #print(output)
        if trange is None: print(good_data, questionable_data, "+", len(output), questionable, "{:.2%}".format(i / len(files)))
        #print(text)
    except AttributeError as e:
        if args.verbose: print(C_RED + f"Error: unable to find transcript in {filename}" + C_ENDC)
        continue
    if len(output) == 0: continue
    title_bad = codecs.decode(codecs.encode(title, "ascii", "ignore"), "ascii") != title
    filename_title = bad_filename_characters.sub("_", title)
    #if title_bad:
    #    print(title)
    #    print( codecs.decode(codecs.encode(title, "ascii", "ignore"), "ascii"))
    newlines = output.count("\n")
    cpl = len(output) / newlines
    for directory in ["junk", "questionable", "check", "good"]:
        if not os.path.exists(directory):
            os.mkdir(directory)
    if title in good_files:
        f = open("good/{}.txt".format(filename_title), "w")
    elif cpl > 300 or cpl < 20 or newlines < 20 or title in bad_files:
        f = open("junk/{}.txt".format(filename_title), "w")
    elif questionable or len(output) < 1000 or title in questionable_files:
        f = open("questionable/{}.txt".format(filename_title), "w")
    #elif title_bad:
    #    f = open("check/{}.txt".format(title), "w")
    else:
        f = open("good/{}.txt".format(filename_title), "w")
    f.write(output)
        
